import React, { useState, useEffect } from 'react';
import {
  DatePicker,
  Input,
  Table,
  Row,
  Col,
  Button,
  notification,
  message,
  Spin,
} from 'antd';
import { SwapRightOutlined } from '@ant-design/icons';
import moment from 'moment';
import { get, CITY_SERVICE } from '@/services/index';
import styles from './index.less';

const OmsLog = () => {
  const [loading, setLoading] = useState(false); // 源数据
  const [data, setData] = useState([]); // 源数据
  const [levelFilters, setLevelFilters] = useState([]); // 优先级筛选
  const [functionName, setFunctionName] = useState(''); // 接口名称筛选
  const [label, setLabel] = useState(''); // 标签筛选
  const [startTime, setStartTime] = useState(moment().startOf('day')); // 默认值当天0点
  const [endTime, setEndTime] = useState(
    moment(new Date(), 'YYYY-MM-DD HH:mm:ss'), // 默认值当前时间
  );

  const columns = [
    {
      title: '操作时间',
      dataIndex: 'logTime',
      key: 'logTime',
      width: 200,
      defaultSortOrder: 'descend',
      sortDirections: ['descend', 'ascend'],
      sorter: (a, b) =>
        new Date(a.logTime).getTime() - new Date(b.logTime).getTime(),
    },
    {
      title: '接口名称',
      dataIndex: 'functionName',
      key: 'functionName',
      width: 200,
      // filters: functionNameFilters,
      // onFilter: (value, record) => record.functionName === value,
    },
    {
      title: '标签',
      dataIndex: 'label',
      key: 'label',
      width: 360,
    },
    {
      title: '优先级',
      dataIndex: 'level',
      key: 'level',
      width: 100,
      filters: levelFilters,
      onFilter: (value, record) => record.level === value,
      render: record => {
        if (record === '高') {
          return <span style={{ color: '#ff7875' }}>高</span>;
        }
        if (record === '中') {
          return <span style={{ color: '#faad14' }}>中</span>;
        }
        if (record === '低') {
          return <span style={{ color: '#52c41a' }}>低</span>;
        }
        return record;
      },
    },
    {
      title: '详情',
      dataIndex: 'shortInfo',
      key: 'shortInfo',
      ellipsis: 'true',
      // width: 300,
    },
  ];
  // 在起止时间任意一个变化后获取数据
  useEffect(() => {
    if (startTime && endTime) {
      setLoading(true);
      getData();
    }
  }, [startTime, endTime]);
  const getData = () => {
    get(`${CITY_SERVICE}/OMS.svc/S_GetOMSLog`, {
      _version: 9999,
      _dc: Date.now(),
      logType: 'operationLog',
      startDate: startTime.format('YYYY-MM-DD'),
      endDate: endTime.format('YYYY-MM-DD'),
      startTime: startTime.format('HH:mm'),
      endTime: endTime.format('HH:mm'),
      function: functionName,
      label,
    })
      .then(res => {
        if (res.success) {
          if (!res.root) {
            setData([]);
          } else {
            setData(res.root);
            // 过滤优先级
            let arr1 = res.root.map(item => item.level);
            arr1 = arr1.filter((value, index) => arr1.indexOf(value) === index);
            setLevelFilters(arr1.map(item => ({ text: item, value: item })));
          }
        } else {
          notification.error({
            message: '数据获取失败',
            description: res.message,
          });
        }
        setLoading(false);
      })
      .catch(err => {
        message.error(err);
        setLoading(false);
      });
  };
  // DatePicker改变点击确定时
  const changeStartTime = time => {
    setStartTime(time);
  };
  const changeEndTime = time => {
    setEndTime(time);
  };
  // 近1/6/12/24小时
  const setTime = time => {
    setEndTime(moment(new Date(), 'YYYY-MM-DD HH:mm:ss'));
    setStartTime(
      moment(
        new Date(new Date().getTime() - time * 60 * 60 * 1000),
        'YYYY-MM-DD HH:mm:ss',
      ),
    );
  };

  return (
    <>
      <div className={styles.omsLog}>
        <Row className={styles.head}>
          <Col span={24}>
            <span style={{ lineHeight: 2 }}>时间：</span>
            <DatePicker
              showTime
              format="YYYY-MM-DD HH:mm:ss"
              placeholder="起始时间"
              value={startTime}
              onChange={changeStartTime}
              allowClear={false}
            />
            <SwapRightOutlined style={{ verticalAlign: '0.125em' }} />
            <DatePicker
              showTime
              format="YYYY-MM-DD HH:mm:ss"
              placeholder="结束时间"
              value={endTime}
              onChange={changeEndTime}
              style={{ marginRight: '10px' }}
              allowClear={false}
            />
            <Button onClick={() => setTime(1)}>近1小时</Button>
            {/* <Button onClick={() => setTime(6)}>近6小时</Button> */}
            <Button onClick={() => setTime(24)}>近1天</Button>
            <Button onClick={() => setTime(24 * 7)}>近1周</Button>
            <Button onClick={() => setTime(14 * 24)}>近2周</Button>
            {/* <Button onClick={() => setTime(30 * 24)}>近1月</Button> */}
            <span style={{ marginLeft: '10px' }}>接口查询：</span>
            <Input
              style={{ width: '120px' }}
              placeholder="请输入接口名称"
              onChange={e => {
                setFunctionName(e.target.value);
              }}
            />
            <span style={{ marginLeft: '10px' }}>标签查询：</span>
            <Input
              style={{ width: '120px' }}
              placeholder="请输入标签"
              onChange={e => {
                setLabel(e.target.value);
              }}
            />
            <Button
              type="primary"
              style={{ marginLeft: '10px' }}
              onClick={getData}
            >
              查询
            </Button>
          </Col>
        </Row>
        <Spin spinning={loading} tip="loading">
          <div className={styles.table}>
            <Table
              size="small"
              bordered
              columns={columns}
              dataSource={data}
              scroll={{ x: 'max-content', y: 'calc(100vh - 208px)' }}
              pagination={{
                showTotal: (total, range) =>
                  `第${range[0]}-${range[1]} 条/共 ${total} 条`,
                pageSizeOptions: [10, 20, 50, 100],
                defaultPageSize: 20,
                showQuickJumper: true,
                showSizeChanger: true,
              }}
            />
          </div>
        </Spin>
      </div>
    </>
  );
};
export default OmsLog;
